@@ -6,13 +6,13 @@ WWW::Mechanize - Handy web browsing in a Perl object
 
 =head1 VERSION
 
-Version 1.03_02
+Version 1.05_01
 
-    $Header: /cvsroot/www-mechanize/www-mechanize/lib/WWW/Mechanize.pm,v 1.133 2004/08/17 04:03:19 petdance Exp $
+    $Header: /cvsroot/www-mechanize/www-mechanize/lib/WWW/Mechanize.pm,v 1.142 2004/10/01 02:16:10 petdance Exp $
 
 =cut
 
-our $VERSION = "1.03_02";
+our $VERSION = "1.05_01";
 
 =head1 SYNOPSIS
 
@@ -105,7 +105,7 @@ use URI::URL;
 
 our @ISA = qw( LWP::UserAgent );
 
-=head1 Constructor and startup
+=head1 CONSTRUCTOR AND STARTUP
 
 =head2 new()
 
@@ -167,6 +167,12 @@ installed, or C<CORE::die> if not.
 Don't complain on warnings.  Setting C<< quiet => 1 >> is the same as
 calling C<< $agent->quiet(1) >>.  Default is off.
 
+=item * C<< stack_depth => $value >>
+
+Sets the depth of the page stack that keeps tracks of all the downloaded
+pages. Default is 0 (infinite). If the stack is eating up your memory,
+then set it to 1.
+
 =back
 
 =cut
@@ -184,6 +190,7 @@ sub new {
         onwarn      => \&WWW::Mechanize::_warn,
         onerror     => \&WWW::Mechanize::_die,
         quiet       => 0,
+        stack_depth => 0,
         headers     => {},
     );
 
@@ -207,6 +214,10 @@ sub new {
     }
     $self->{page_stack} = [];
     $self->env_proxy();
+
+    # libwww-perl 5.800 (and before, I assume) has a problem where
+    # $ua->{proxy} can be undef and clone() doesn't handle it.
+    $self->{proxy} = {} unless defined $self->{proxy};
     push( @{$self->requests_redirectable}, 'POST' );
 
     $self->_reset_page;
@@ -278,12 +289,13 @@ sub known_agent_aliases {
     return sort keys %known_agents;
 }
 
-=head1 Page-fetching methods
+=head1 PAGE-FETCHING METHODS
 
-=head2 $mech->get($url)
+=head2 $mech->get( $url )
 
 Given a URL/URI, fetches it.  Returns an L<HTTP::Response> object.
-I<$url> can be a well-formed URL string, or a L<URI> object.
+I<$url> can be a well-formed URL string, a L<URI> object, or a
+L<WWW::Mechanize::Link> object.
 
 The results are stored internally in the agent object, but you don't
 know that.  Just use the accessors listed below.  Poking at the internals
@@ -303,6 +315,8 @@ sub get {
     my $self = shift;
     my $uri = shift;
 
+    $uri = $uri->url if ref($uri) eq 'WWW::Mechanize::Link';
+
     $uri = $self->base
             ? URI->new_abs( $uri, $self->base )
             : URI->new( $uri );
@@ -397,7 +411,7 @@ sub follow_link {
     return $response;
 }
 
-=head1 Form field filling methods
+=head1 FORM FIELD FILLING METHODS
 
 =head2 $mech->form_number($number)
 
@@ -579,7 +593,7 @@ sub set_visible {
     my $form = $self->current_form;
     my @inputs = $form->inputs;
 
-    while (my $value = shift) {
+    for my $value ( @_ ) {
         if ( ref $value eq 'ARRAY' ) {
            my ( $type, $value ) = @$value;
            while ( my $input = shift @inputs ) {
@@ -596,7 +610,7 @@ sub set_visible {
                last;
            } # while
        }
-    } # while
+    } # for
 
 } # set_visible()
 
@@ -647,7 +661,7 @@ sub untick {
     shift->tick(shift,shift,undef);
 }
 
-=head1 Form submission methods
+=head1 FORM SUBMISSION METHODS
 
 =head2 $mech->click( $button [, $x, $y] )
 
@@ -856,7 +870,7 @@ sub submit_form {
     return $response;
 }
 
-=head1 Status methods
+=head1 STATUS METHODS
 
 =head2 $mech->success()
 
@@ -965,7 +979,7 @@ sub title {
     return $p->header('Title');
 }
 
-=head1 Content-handling methods
+=head1 CONTENT-HANDLING METHODS
 
 =head2 $mech->find_link()
 
@@ -1166,7 +1180,7 @@ sub find_all_links {
 }
 
 
-=head1 Miscellaneous methods
+=head1 MISCELLANEOUS METHODS
 
 =head2 $mech->add_header( name => $value [, name => $value... ] )
 
@@ -1249,7 +1263,21 @@ sub quiet {
     return $self->{quiet};
 }
 
-=head1 Overridden L<LWP::UserAgent> methods
+=head2 $mech->stack_depth($value)
+
+Get or set the page stack depth. Older pages are discarded first.
+
+A value of 0 means "keep all the pages".
+
+=cut
+
+sub stack_depth {
+    my $self = shift;
+    $self->{stack_depth} = shift if @_;
+    return $self->{stack_depth};
+}
+
+=head1 OVERRIDDEN L<LWP::UserAgent> METHODS
 
 =head2 $mech->redirect_ok()
 
@@ -1380,7 +1408,11 @@ sub _modify_request {
         $req->header( 'Accept-Encoding', 'identity' );
     }
 
-    $req->header( Referer => $self->{last_uri} ) if $self->{last_uri};
+    my $last = $self->{last_uri};
+    if ( $last ) {
+        $last = $last->as_string if ref($last);
+        $req->header( Referer => $last );
+    }
     while ( my($key,$value) = each %{$self->{headers}} ) {
         if ( defined $value ) {
             $req->header( $key => $value );
@@ -1405,7 +1437,7 @@ sub _make_request {
     $self->SUPER::request(@_);
 }
 
-=head1 Deprecated methods
+=head1 DEPRECATED METHODS
 
 This methods have been replaced by more flexible and precise methods.
 Please use them instead.
@@ -1472,7 +1504,7 @@ sub form {
     return $arg =~ /^\d+$/ ? $self->form_number($arg) : $self->form_name($arg);
 }
 
-=head1 Internal-only methods
+=head1 INTERNAL-ONLY METHODS
 
 These methods are only used internally.  You probably don't need to
 know about them.
@@ -1519,25 +1551,33 @@ sub _extract_links {
 
     while (my $token = $p->get_tag( keys %urltags )) {
         my $tag = $token->[0];
-        my $url = $token->[1]{$urltags{$tag}};
+		my $attrs = $token->[1];
+        my $url = $attrs->{$urltags{$tag}};
 
         my $text;
         my $name;
+		my $alt;
         if ( $tag eq "a" ) {
             $text = $p->get_trimmed_text("/$tag");
             $text = "" unless defined $text;
 
-            my $onClick = $token->[1]{onclick};
+            my $onClick = $attrs->{onclick};
             if ( $onClick && ($onClick =~ /^window\.open\(\s*'([^']+)'/) ) {
                 $url = $1;
             }
         } # a
-        if ( $tag ne "area" ) {
-            $name = $token->[1]{name};
+
+		# Of the tags we extract from, only 'AREA' has an alt tag
+		if ($tag eq 'area') {
+			$alt = $attrs->{alt};
+		}
+		# The rest should have a 'name' attribute.
+		else  {
+            $name = $attrs->{name};
         }
         if ( $tag eq "meta" ) {
-            my $equiv = $token->[1]{"http-equiv"};
-            my $content = $token->[1]{"content"};
+            my $equiv = $attrs->{"http-equiv"};
+            my $content = $attrs->{"content"};
             next unless $equiv && (lc $equiv eq "refresh") && defined $content;
 
             if ( $content =~ /^\d+\s*;\s*url\s*=\s*(.+)/ ) {
@@ -1548,7 +1588,14 @@ sub _extract_links {
         } # meta
 
         next unless defined $url;   # probably just a name link or <AREA NOHREF...>
-        push( @{$self->{links}}, WWW::Mechanize::Link->new( $url, $text, $name, $tag, $self->base ) );
+        push( @{$self->{links}}, WWW::Mechanize::Link->new({
+				url => $url,
+				text => $text,
+				name => $name,
+				tag => $tag,
+				base => $self->base,
+				alt => $alt,
+			}) );
     } # while
 
     # Old extract_links() returned a value.  Carp if someone expects
@@ -1583,7 +1630,11 @@ sub _push_page_stack {
         $self->{page_stack} = [];
 
         push( @$save_stack, $self->clone );
-
+        if ( $self->stack_depth > 0 ) {
+            while ( @$save_stack > $self->stack_depth ) {
+                shift @$save_stack;
+            }
+        } # if stack_depth > 0
         $self->{page_stack} = $save_stack;
     }
 
@@ -1736,10 +1787,14 @@ Here are modules that use or subclass Mechanize.  Let me know of any others:
 
 =over 4
 
+=item * L<Finance::Bank::LloydsTSB>
+
 =item * L<WWW::Bugzilla>
 
 =item * L<WWW::Google::Groups>
 
+=item * L<WWW::Hotmail>
+
 =item * L<WWW::Mechanize::Cached>
 
 =item * L<WWW::Mechanize::FormFiller>
@@ -1758,7 +1813,7 @@ Here are modules that use or subclass Mechanize.  Let me know of any others:
 
 =back
 
-=head1 Requests & Bugs
+=head1 REQUESTS & BUGS
 
 Please report any requests, suggestions or (gasp!) bugs via the
 excellent RT bug-tracking system at http://rt.cpan.org/, or email to
@@ -1768,13 +1823,40 @@ track things.
 L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=WWW-Mechanize> is the RT queue
 for Mechanize.  Please check to see if your bug has already been reported.
 
-=head1 Author
+=head1 AUTHOR
 
 Copyright 2004 Andy Lester <andy@petdance.com>
 
 Released under the Artistic License.  Based on Kirrily Robert's excellent
 L<WWW::Automate> package.
 
+=head1 ACKNOWLEDGEMENTS
+
+Thanks to the numerous people who have helped out on WWW::Mechanize in
+one way or another, including
+Kirrily Robert,
+Mike O'Regan,
+Mark Stosberg,
+Uri Guttman,
+Peter Scott,
+Phillipe Bruhat,
+Ian Langworth,
+John Beppu,
+Gavin Estey,
+Jim Brandt,
+Ask Bjoern Hansen,
+Greg Davies,
+Ed Silva,
+Mark-Jason Dominus,
+Autrijus Tang,
+Mark Fowler,
+Stuart Children,
+Max Maischein,
+Meng Wong,
+Prakash Kailasa,
+Abigail,
+and the late great Iain Truskett.
+
 =cut
 
 1;
